@@ -10,7 +10,7 @@ use warnings;
 
 use Carp;
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 =head1 NAME
 
@@ -46,6 +46,14 @@ sub new
    return bless {}, $class;
 }
 
+# We're using this internally in gkpar() but not documenting it currently.
+# Details are still experimental.
+sub enter
+{
+   my $self = shift;
+   $self->again( @_ );
+}
+
 # Keep perl happy; keep Britain tidy
 1;
 
